//////////////////////////////////////////////////////////////
/////           Divers schemas de volumes finis        ///////
/////               pour pbs elliptiques 1D            ///////
/////                                                  ///////
/////        Definition et outils pour les maillages   ///////
/////                                                  ///////
/////                F. Boyer  &  S. Krell             ///////
/////                     Mars 2010                    ///////
//////////////////////////////////////////////////////////////


//**************************************************************************
//    Le format des maillages
//**************************************************************************
//  Chaque maillage est une structure contenant 5 elements :
//  *  maillage.nom     :  le nom de la famille de maillages
//  *  
//  *  maillage.nb_vol  :  le nombre de volumes de contrôle
//  *  maillage.centres :  vecteur de taille nb_vol contenant la position 
//                         des centres des volumes de contrôle
//  *  maillage.mes     :  vecteur de taille nb_vol contenant la mesure
//                         des volumes de contrôle
//  *  maillage.sommets :  vecteur de taille nb_vol+1 contenant la position
//                         des sommets
//         maillage.sommets(i) correspond à x_{i-1/2} pour 1<=i<=nb_vol+1
//  *  maillage.dist    :  vecteur de taille nb_vol+1 contenant la distance
//                         entres les centres de deux volumes voisins
//         maillage.dist(i) correspond à h_{i-1/2} pour 1<=i<=nb_vol+1
//**************************************************************************



//**************************************************************************
//   Fonctions de creation de maillages
//**************************************************************************
//
//  REMARQUE : pour des raisons techniques, on a imposé pour tous les maillages
//  que le milieu de l'intervalle ]0,1[ soit un sommet du maillage
//
//**************************************************************************


//*********  Maillages uniformes

function [maillage] = maillage_uniforme(N);
  
  h=1/N;
  sommets=h*[0:N]';
  centres=h*[1:N]'-h/2;
  mes=h.*ones(N,1);
  dist=h.*ones(N+1,1);
  dist(1)=h/2;
  dist(N+1)=h/2;
  
  maillage=struct("nom","maillage uniforme",...
		  "sommets",sommets,...
		  "centres",centres,...
		  "mes",mes,...
		  "dist",dist,...
		  "nb_vol",length(centres));
  
endfunction

//********** Maillages alternes 

function [maillage] = maillage_alterne(N);
  
  // On s'arrange pour que le nombre de mailles soit un multiple de 4
  // ce qui assure que le point x=0.5 est un sommet du maillage
  N=(int((N-0.1)/4)+1)*4;
  
  x=ones(N,1);
  x(1:2:$)=2;
  x=cumsum(x);
  x=x/max(x);

  sommets=[0; x];
  centres=zeros(N,1);
  mes=zeros(N,1);
  dist=zeros(N+1,1);
  for i=1:N
    centres(i)=(sommets(i+1)+ sommets(i) )/2;
  end  
  for i=1:N
    mes(i)=sommets(i+1)- sommets(i);
  end  
  for i=2:N
    dist(i)=centres(i)- centres(i-1);
  end
  dist(1)=centres(1);
  dist(N+1)=1-centres(N);
  
  maillage=struct("nom","maillage alterne",...
		  "sommets",sommets,...
		  "centres",centres,...
		  "mes",mes,...
		  "dist",dist,...
		  "nb_vol",length(centres));
  
endfunction


//********** Maillages aleatoires (perturbation aleatoire d'un maillage uniforme)

function [maillage] = maillage_aleatoire(N);
  
  x=linspace(0+(1/(N-1)),1-1/(N-1),N-1)'+0.1*(2*rand(N-1,1)-1)/(N-1);
  
  x=gsort(x,'g','i');
  sommets = [ 0; x; 1 ];
  
  // On s'arrange pour que 0.5 soit un sommet du maillage
  [val,indice]=min(abs(sommets-0.5));
  indice=min(indice); 
  sommets(indice)=0.5;
  
  centres=zeros(N,1);
  mes=zeros(N,1);
  dist=zeros(N+1,1);
  for i=1:N
    centres(i)=(sommets(i+1)+ sommets(i) )/2;
  end  
  for i=1:N
    mes(i)=sommets(i+1)- sommets(i);
  end  
  for i=2:N
    dist(i)=centres(i)- centres(i-1);
  end
  dist(1)=centres(1);
  dist(N+1)=1-centres(N);
  
  maillage=struct("nom","maillage aleatoire",...
		  "sommets",sommets,...
		  "centres",centres,...
		  "mes",mes,...
		  "dist",dist,...
		  "nb_vol",length(centres));
  
endfunction


//********** Maillages étirés près du bord et raffinés près du centre

function [maillage] = maillage_stretch(N);
  alp=100/101;//9/10;//3/4;
  n=floor(N/2);
  sommets=zeros(N+1,1);
  centres=zeros(N,1);
  mes=zeros(N,1);
  dist=zeros(N+1,1);
  for i=1:n+1
    sommets(i)=(1-alp^(i-1))/(2*(1-alp^(n)))
  end
  for i=n+2:N+1
    sommets(i)=1-(1-alp^(N-i+1))/(2*(1-alp^(N-n)))
  end
  for i=1:N
    centres(i)=(sommets(i+1)+ sommets(i))/2;
  end  
  for i=1:N
    mes(i)=sommets(i+1)- sommets(i);
  end  
  for i=2:N
    dist(i)=centres(i)- centres(i-1);
  end
  dist(1)=centres(1);
  dist(N+1)=1-centres(N);
  
  maillage=struct("nom","maillage stretch",....
		  "sommets",sommets,...
		  "centres",centres,...
		  "mes",mes,...
		  "dist",dist,...
		  "nb_vol",length(centres));
  
endfunction


//**************************************************************************
//   Fonctions d'affichage d'un maillage
//**************************************************************************
//   A n'utiliser que pour des petits nombres de volumes de contrôle
//**************************************************************************

function trace_maillage(m);
  plot(m.centres,0*m.centres,'bs');
  e=gce();
  e.children.mark_size=10;
  plot(m.sommets,0*m.sommets,'rd');
  e=gce();
  e.children.mark_size=10;
endfunction

//**************************************************************************
//   Calculs de normes discrètes
//**************************************************************************

function [norme]=normeL2(m,u)
  norme=sqrt(sum(m.mes.*(u.^2)));
endfunction

// Pour calculer le gradient, on suppose que la fonction est nulle au bord
// Ce n'est pas gênant car ces fonctions sont destinées à être appliquées
// a la difference entre la solution exacte et la solution approchee.

function [norme]=normeH1(m,u)
  gu=([u; 0]-[0; u])./m.dist;
  norme=sqrt(sum(m.dist.*(gu.^2)));
endfunction

function [norme]=normeLip(m,u)
  gu=([u; 0]-[0; u])./m.dist;
  norme=max(abs(gu));
endfunction