//////////////////////////////////////////////////////////////
/////           Divers schemas de volumes finis        ///////
/////               pour pbs elliptiques 1D            ///////
/////                                                  ///////
/////              différents schémas de VF            ///////
/////                                                  ///////
/////                F. Boyer  &  S. Krell             ///////
/////                     Mars 2010                    ///////
//////////////////////////////////////////////////////////////

// Toutes ces fonctions prennent deux paramètres :
//      m        : un maillage
//      donnees  : un jeu de donnees
//

//**************************************************************************
//    Le schéma de base pour le Laplacien 
//    avec conditions de Dirichlet homogène
//**************************************************************************
function [A,b]=vf_laplacien_dirh(m,donnees);

  printf('Assemblage du schéma VF - Laplacien Dirichlet homogène - %g mailles\n',m.nb_vol);
    
  A=spzeros(m.nb_vol,m.nb_vol);
  b=zeros(m.nb_vol,1);
  
  for i=2:m.nb_vol-1
    A(i,i)= (1/m.dist(i)+1/m.dist(i+1));
    A(i,i-1)=- 1/m.dist(i);
    A(i,i+1)=- 1/m.dist(i+1);
  end  
  
  A(1,1)=(1/m.dist(1)+1/m.dist(2));
  A(1,2)=-1/m.dist(2);
  A(m.nb_vol,m.nb_vol)= (1/m.dist(m.nb_vol+1)+1/m.dist(m.nb_vol));
  A(m.nb_vol,m.nb_vol-1)= - 1/m.dist(m.nb_vol);

  b=feval(m.centres,donnees.source).*m.mes;


endfunction

//**************************************************************************
//    Le schéma de base pour le Laplacien 
//    avec conditions de Dirichlet non homogène
//**************************************************************************
function [A,b]=vf_laplacien_dirnh(m,donnees);

  printf('Assemblage du schéma VF - Laplacien Dirichlet non homogène - %g mailles\n',m.nb_vol);
  
  [A,b]=vf_laplacien_dirh(m,donnees);
  
  dir_g=donnees.bordD(0);
  dir_d=donnees.bordD(1);
  
  // Prise en compte des donnees de Dirichlet au bord
  b(1)=b(1)+1/m.dist(1)*dir_g;
  b(m.nb_vol)=b(m.nb_vol)+1/m.dist(m.nb_vol+1)*dir_d;

endfunction

//**************************************************************************
//    Routine de calcul du coefficient de diffusion aux interfaces
//        selon la méthode choisie
//**************************************************************************
function [ck]=calcul_coeff_k_interf(m,donnees);
  
  if ~isfield(donnees,"methode") then
    donnees.methode="exacte";
  end;
  
  select donnees.methode
   case "exacte"  // On fait le calcul exact du coefficient aux sommets
                  // du maillage
    
     ck=feval(m.sommets,donnees.coeff_k);
     
   case "arithmetique"   // On fait la moyenne arithmétique des valeurs du
                  // coefficient aux centres des mailles voisines
  
     ck_c=feval(m.centres,donnees.coeff_k); // Les valeurs aux centres
     ck=0.5*(ck_c(2:$)+ck_c(1:$-1));  // Calcul pour les interfaces internes
     ck=[ck_c(1); ck; ck_c($)];         // On rajoute les valeurs au bord
     
   case "harmonique"   // On fait la moyenne harmonique pondérée des
                       // valeurs du coefficient aux centres des mailles voisines
  
     ck_c=feval(m.centres,donnees.coeff_k); // Les valeurs aux centres
     ck=m.dist(2:$-1).*ck_c(2:$).*ck_c(1:$-1)./...
           ((m.sommets(2:$-1)-m.centres(1:$-1)).*ck_c(2:$) ...
          + (m.centres(2:$)-m.sommets(2:$-1)).*ck_c(1:$-1));
     ck=[ck_c(1); ck; ck_c($)];         // On rajoute les valeurs au bord    
  else
     disp("Vous n''avez pas choisi une methode de calcul du coeff de"+...
	  " diffusion valable!");
     abort;
  end;
endfunction

//**************************************************************************
//    Le schéma général pour le problème de diffusion
//    avec conditions de Dirichlet non homogène
//       - prise en compte des divers calculs du coefficient de diffusion
//             aux interfaces
//       - prise en compte d'un éventuel terme de Dirac
//**************************************************************************
function [A,b]=vf_diffusion_dirnh(m,donnees);

  printf('Assemblage du schéma VF - Diffusion Dirichlet non homogène - %g mailles\n',m.nb_vol);
  
  dir_g=donnees.bordD(0);
  dir_d=donnees.bordD(1);
  
  ck=calcul_coeff_k_interf(m,donnees);
    
  A=spzeros(m.nb_vol,m.nb_vol);
  b=zeros(m.nb_vol,1);
  
  for i=2:m.nb_vol-1
    A(i,i)= (ck(i)/m.dist(i)+ck(i+1)/m.dist(i+1));
    A(i,i-1)=- ck(i)/m.dist(i);
    A(i,i+1)=- ck(i+1)/m.dist(i+1);
  end
  A(1,1)=  (ck(1)/m.dist(1)+ck(2)/m.dist(2));
  A(1,2)= - ck(2)/m.dist(2);
  A(m.nb_vol,m.nb_vol)=  (ck(m.nb_vol+1)/m.dist(m.nb_vol+1)+ck(m.nb_vol)/m.dist(m.nb_vol));
  A(m.nb_vol,m.nb_vol-1) = - ck(m.nb_vol)/m.dist(m.nb_vol);

  // Assemblage du terme source
  
  b=feval(m.centres,donnees.source).*m.mes;
 
  // Prise en compte des donnees de Dirichlet au bord
  
  b(1)=b(1)+ck(1)/m.dist(1)*dir_g;
  b(m.nb_vol)=b(m.nb_vol)+ck(m.nb_vol+1)/m.dist(m.nb_vol+1)*dir_d;

 
  /// On rajoute le terme de dirac si necessaire
 
  
  if (isfield(donnees,'dirac')) then

    // on cherche i tel que  centres(i)<= dirac < centres(i+1)
    // Attention cela ne marche que si le Dirac n'est pas situé dans les mailles du bord.
    
    ind=find(donnees.dirac>=m.centres);
    i=max(ind);
            
    b(i)   = b(i)    +  (m.centres(i+1)-donnees.dirac)/m.dist(i+1); 
    b(i+1) = b(i+1)  +  (donnees.dirac-m.centres(i))/m.dist(i+1);
     
  end;  
  
  
endfunction
  
//**************************************************************************
//    Le schéma général pour le problème de diffusion
//    avec conditions de Neumann non homogène
//       - prise en compte des divers calculs du coefficient de diffusion
//             aux interfaces
//       - prise en compte d'un éventuel terme de Dirac
//**************************************************************************
  
function [A,b]=vf_diffusion_neunh(m,donnees);
 
  printf('Assemblage du schéma VF - Diffusion - Neumann non homogène - %g mailles\n',m.nb_vol);
  
  if ~isfield(donnees,"bordN") then
     printf("Pas de donnee au bord de type Neumann !");
     abort;
  end;

  if ~isfield(donnees,"moyenne") then
     printf("Pour un calcul de Neumann il faut fixer la moyenne !\n");
     abort;
  end;
  
  ck=calcul_coeff_k_interf(m,donnees);
  
  A=spzeros(m.nb_vol,m.nb_vol);
  b=zeros(m.nb_vol,1);
  
  for i=2:m.nb_vol-1
    A(i,i)= (ck(i)/m.dist(i)+ck(i+1)/m.dist(i+1));
    A(i,i-1)=- ck(i)/m.dist(i);
    A(i,i+1)=- ck(i+1)/m.dist(i+1);
  end  
  A(1,1)=  ck(2)/m.dist(2)  ;
  A(1,2)= - ck(2)/m.dist(2);
  A(m.nb_vol,m.nb_vol)=  ck(m.nb_vol)/m.dist(m.nb_vol)  ;
  A(m.nb_vol,m.nb_vol-1) = - ck(m.nb_vol)/m.dist(m.nb_vol);
  
  // Assemblage du terme source
  
  b=feval(m.centres,donnees.source).*m.mes;
  
  // bordN renvoie les flux sortant i.e.  :  - k du/dn au bord
  
  b(1)=b(1)+donnees.bordN(0);
  b(m.nb_vol)=b(m.nb_vol)+donnees.bordN(1);
  
  /// On rajoute le terme de dirac si necessaire
  
  if (isfield(donnees,'dirac')) then

    // on cherche i tel que  centres(i)<= dirac < centres(i+1)
    // Attention cela ne marche que si le Dirac n'est pas situé dans les mailles du bord.
    
    ind=find(donnees.dirac>=m.centres);
    i=max(ind);
            
    b(i)   = b(i)    +  (m.centres(i+1)-donnees.dirac)/m.dist(i+1); 
    b(i+1) = b(i+1)  +  (donnees.dirac-m.centres(i))/m.dist(i+1);  
    
  end;
  
  /// On impose la moyenne par penalisation
  
  epsilon=10^(-10);
  A(1,:)=A(1,:)+m.mes'/epsilon;
  b(1)=b(1)+donnees.moyenne/epsilon;
  
endfunction