//////////////////////////////////////////////////////////////
/////           Divers schemas de volumes finis        ///////
/////               pour pbs elliptiques 2D            ///////
/////                                                  ///////
/////                  Jeux de données                 ///////
/////                                                  ///////
/////                F. Boyer  &  S. Krell             ///////
/////                     Mars 2010                    ///////
//////////////////////////////////////////////////////////////

//**************************************************************************
//    Le format des jeux de données
//**************************************************************************
//  Chaque jeu de données contient (ou pas !) un certain nombre d'éléments
//  *  nom :  chaîne de caractère qui décrit le cas test
//  *  source    : fonction (x,y)->f(x,y) qui décrit le terme source
//  *  uexacte   : fonction (x,y)->ue(x,y) qui donne la solution exacte, si elle existe
//  *  coeff_k   : fonction (x,y)->k(x,y) qui décrit le coefficient de diffusion isotrope
//  *  coeff_mat : structure contenant trois fonctions qui décrivent un tenseur de diffusion
//  *  bordD     : fonction (x,y)->borD(x,y) qui renvoie les données de Dirichlet au bord
//  *  methode   : choix de la methode de calcul du coefficient ou tenseur de diffusion 
//                 à l'interface entre deux volumes de contrôle
//  *  maillages : liste de noms de maillages utilisables pour ce cas_test
//                 des listes sont prédéfinies dans maillages2D.sci
//  *  beta  : le coefficient qui intervient dans le modèle de fractures
//  *  courant : nombre de fonctions de courant à afficher (0 pour ne pas en tracer)
////////////////////////////////////////////////////////////////////////////


//**************************************************************************
//   Un produit de sinus pour le laplacien
//**************************************************************************

function [z]=f_nulle(x,y)
  z=0;
endfunction

function [z]=coeff_cte(x,y)
  z=1;
endfunction


cas_test=list();

function [z]=f1(x,y)
  z=2*%pi^2*sin(%pi*x)*sin(%pi*y);
endfunction

function [z]=ue1(x,y)
  z=sin(%pi*x)*sin(%pi*y);
endfunction

cas_test($+1)=struct("nom","u(x,y)=sin(pi*x)sin(pi*y)",...
		     "coeff_k",coeff_cte,...
		     "uexacte",ue1,...
		     "source",f1,...
		     "bordD",ue1,...
                     "maillages",maillages_carre_Dir);


//**************************************************************************
//   Un polynome pour le laplacien
//**************************************************************************

function [z]=f2(x,y)
  z=32*y.*(1-y)+32*x.*(1-x);
endfunction

function [z]=ue2(x,y)
  z=16*x.*(1-x)*y.*(1-y);
endfunction

cas_test($+1)=struct("nom","u(x,y)=16x(1-x)y(1-y)",...
		     "coeff_k",coeff_cte,...
		     "uexacte",ue2,...
		     "source",f2,...
		     "bordD",ue2,...
                     "maillages",maillages_carre_Dir);

//**************************************************************************
//   Une solution du laplacien non nulle au bord
//**************************************************************************

function [z]=f3(x,y)
  dxx=-(1-y).^2.*sin((1-x).*(1-y))+6*(1-x).*(1-y).^2;
  dyy=-(1-x).^2.*sin((1-x).*(1-y))+2*(1-x).^3;
  z=-dxx-dyy;
endfunction

function [z]=ue3(x,y)
  z=sin((1-x).*(1-y))+(1-x).^3.*(1-y).^2;
endfunction

cas_test($+1)=struct("nom","u(x,y)=sin((1-x)(1-y))+(1-x)^3(1-y)^2",...
		     "coeff_k",coeff_cte,...
		     "uexacte",ue3,...
		     "source",f3,...
		     "bordD",ue3,...
                     "maillages",maillages_carre_Dir);


//**************************************************************************
//   Ecoulement vertical dans un milieu poreux fracture
//   donnée au bord est donnée par (x,y)  -->  y
//**************************************************************************

function [z]=ubord2(x,y)
  z=y;
endfunction


cas_test($+1)=struct("nom","Ecoulement vertical dans un reseau de fractures",...
		     "source",f_nulle,...
		     "coeff_k",coeff_cte,...
		     "bordD",ubord2,...
		     "bordN",f_nulle,...
		     "beta",0,...
		     "courant",40,...
		     "maillages",maillages_fractures);
		     
//**************************************************************************
//  La solution est un produit de sinus
//  mais le coefficient de diffusion est matriciel et représente un champ tournant
//**************************************************************************

function [z]=mat_xx_cont(x,y)
  z=(10^(-3)*x.^2+y.^2)./(x.^2+y.^2);
endfunction
function [z]=mat_yy_cont(x,y)
  z=(x.^2+10^(-3)*y.^2)./(x.^2+y.^2);
endfunction
function [z]=mat_xy_cont(x,y)
  z=(10^(-3)-1)*x.*y./(x.^2+y.^2);
endfunction
function [z]=f5(x,y)
  u_1=(10^(-3)*x.^2+y.^2)*%pi*cos(%pi*x).*sin(%pi*y) ...
  +%pi*sin(%pi*x)*cos(%pi*y).*(10^(-3)-1).*x.*y;
  u_2=(10^(-3)-1)*x.*y*%pi*cos(%pi*x).*sin(%pi*y) ...
  +%pi*sin(%pi*x)*cos(%pi*y).*(x.^2+10^(-3)*y.^2);
  
  u_1p=10^(-3)*2*x.*%pi*cos(%pi*x).*sin(%pi*y) ...
  -(10^(-3)*x.^2+y.^2)*%pi*%pi.*sin(%pi*x).*sin(%pi*y) ...
  +%pi*sin(%pi*x)*cos(%pi*y).*(10^(-3)-1).*y ...
  +%pi*%pi*cos(%pi*x)*cos(%pi*y).*(10^(-3)-1).*x.*y;
  
  u_2p=(10^(-3)-1)*x.*%pi*cos(%pi*x).*sin(%pi*y) ...
  +(10^(-3)-1)*x.*y.*%pi*%pi*cos(%pi*x).*cos(%pi*y) ...
  -%pi*%pi*sin(%pi*x)*sin(%pi*y).*(x.^2+10^(-3)*y.^2) ...
  +%pi*sin(%pi*x)*cos(%pi*y).*10^(-3)*2.*y;
  
  z=-u_1p./(x.^2+y.^2+10^(-16))+2*u_1.*x./(x.^2+y.^2+10^(-16)).^2 ...
   -u_2p./(x.^2+y.^2+10^(-16))+2*u_2.*y./(x.^2+y.^2+10^(-16)).^2;
endfunction


cas_test($+1)=struct("nom","u(x,y)=sin(pi*x)sin(pi*y) anisotrope",...
		     "coeff_k",coeff_cte,...
		     "uexacte",ue1,...
		     "source",f5,...
		     "bordD",ue1,...
		     "coeff_mat",struct("Axx",mat_xx_cont,"Axy",mat_xy_cont,"Ayy",mat_yy_cont), ...
		     "methode","exacte",...
		     "maillages",maillages_carre_Dir);
	

//**************************************************************************
//  La solution est polynomiale par morceaux
//  et le coefficient de diffusion est isotrope mais discontinu à travers
//  la ligne x=0.5
//**************************************************************************

function [z]=coeff_dis(x,y)
  if (x>0.5) then
    z=1;
  else
    z=5;//15;
  end
endfunction
 
function [z]=ue6bis(x,y)
  const=1/(coeff_dis(0,0));
  if (x>0.5) then
    z=-0.5*(x-const*y+(const-1)*0.5)^2;
  else
    z=-0.5*(const*(x-y))^2;
  end;
endfunction

function [z]=f6bis(x,y)
 const=1/(coeff_dis(0,0));;
  if (x>0.5) then
    z=1+const^2;
  else
    z=(2*coeff_dis(0,0))*const^2;
  end;
endfunction


cas_test($+1)=struct("nom","u(x,y)=polynomiale isotrope discontinue",...
		     "uexacte",ue6bis,...
		     "source",f6bis,...
		     "bordD",ue6bis,...
		     "coeff_k",coeff_dis,...
		     "methode","exacte",...
		     "maillages",maillages_carre_Dir);		


//**************************************************************************
//  La solution est polynomiale par morceaux
//  et le coefficient de diffusion est anisotrope et discontinu à travers
//  la ligne x=0.5
//**************************************************************************

function [z]=mat_xx_dis(x,y)
  if (x>0.5) then
    z=1;
  else
    z=5;//15;
  end
endfunction
function [z]=mat_yy_dis(x,y)
  if (x>0.5) then
    z=1;
  else
    z=4;//40;
  end
endfunction
function [z]=mat_xy_dis(x,y)
  if (x>0.5) then
    z=0;
  else
    z=3;//20;
  end
endfunction
 
function [z]=ue6(x,y)
  const=1/(mat_xx_dis(0,0)-mat_xy_dis(0,0));
  if (x>0.5) then
    z=-0.5*(x-const*y+(const-1)*0.5)^2;
  else
    z=-0.5*(const*(x-y))^2;
  end;
  
endfunction

function [z]=f6(x,y)
 const=1/(mat_xx_dis(0,0)-mat_xy_dis(0,0));
  if (x>0.5) then
    z=1+const^2;
  else
    z=(mat_xx_dis(0,0)-2*mat_xy_dis(0,0)+mat_yy_dis(0,0))*const^2;
  end;
endfunction


cas_test($+1)=struct("nom","u(x,y)=polynomiale anisotrope discontinue",...
		     "uexacte",ue6,...
		     "source",f6,...
		     "bordD",ue6,...
		     "coeff_mat",struct("Axx",mat_xx_dis,"Axy",mat_xy_dis,"Ayy",mat_yy_dis), ...
		     "methode","exacte",...
		     "maillages",maillages_carre_Dir);		