//////////////////////////////////////////////////////////////
/////           Divers schemas de volumes finis        ///////
/////               pour pbs elliptiques 2D            ///////
/////                                                  ///////
/////        Definition et outils pour les maillages   ///////
/////                                                  ///////
/////                F. Boyer  &  S. Krell             ///////
/////                     Mars 2010                    ///////
//////////////////////////////////////////////////////////////

rep_maillages='./maillages/';   // Le répertoire dans lequel sont
                                // stockés les maillages

//**************************************************************************
//    Le format des maillages
//**************************************************************************
//  Chaque maillage est une structure contenant 5 elements :
//  *  maillage.nom     :  le nom de la famille de maillages
//  *  indice_min et indice_max : les niveaux de raffinement maximum disponibles
//               si ces champs ne sont pas définis, il s'agit d'un maillage unique
//  *  maillage.nb_vol  :  le nombre de volumes de contrôle
//  *  maillage.nb_are  :  le nombre d'aretes
//  *  maillage.nb_som  :  le nombre de sommets
//  *  maillage.centres :  matrice de taille nb_volx2 contenant les coordonnées
//                         des centres des volumes de contrôle
//  *  maillage.sommets :  matrice de taille nb_somx2 contenant les coordonnées
//                         des sommets
//  *  maillage.aretes  :  matrice de taille nb_somx17 contenant des infos sur
//                         les aretes / diamants
//                         On décrit ci-dessous la signification de chaque colonne
//**************************************************************************

// Signification des colonnes
//   Pour la lisibilité du code, il faut utiliser ces constantes prédéfinies
//   et non pas les valeurs numériques !!!!

_X=1;   // Coordonnée x         
_Y=2;   // Coordonnée y

_DEB=1; // Numéro du premier sommet de l'arete
_FIN=2; // Numéro du second sommet de l'arete

_K=3;   // Numéro du premier volume voisin
_L=4;   // Numéro du second volume voisin (par convention ceci est <=0
        // pour les aretes du bord

_MES=5; // Mesure de l'arete

_DKsigma=6;  // Distance d_K,sigma
_DLsigma=7;  // Distance d_L,sigma
_DKL=8;      // Distance d_K,L

_MES_K_DEB=9;  // Mesure du quart de diamant commun à K et DEB
_MES_K_FIN=10; // Mesure du quart de diamant commun à K et FIN
_MES_L_DEB=11; // Mesure du quart de diamant commun à L et DEB
_MES_L_FIN=12; // Mesure du quart de diamant commun à L et FIN
               // La somme de ces quatre aires donne l'aire totale du diamant

_N_KL_X=13;    // Coordonnée x de la normale unitaire à l'arete dirigée
               // de K vers L
_N_KL_Y=14;    // Coordoonée y 

_NS_DEBFIN_X=15;  // Coordonnée x de la normale unitaire à l'arete
                  // duale dirigée de DEB vers FIN
_NS_DEBFIN_Y=16;  // Coordonnée y

_LABEL=17;       // Label associée à l'arete
                 // -1 : les aretes du bord dirichlet
                 //  <-1 : le reste du bord
                 //  1 : les aretes correspondant aux fractures

liste_maillages=list();

/////////////////////////////////////////////////////////
/////   Les maillages du carre unite avec CL de Dirichlet
/////////////////////////////////////////////////////////

maillages_carre_Dir=list();

liste_maillages($+1)=struct("nom","maillage_rectangle_uniforme",...
			   "indice_min",1,...
			   "indice_max",6);
maillages_carre_Dir($+1)=liste_maillages($).nom;

liste_maillages($+1)=struct("nom","maillage_rectangle_bidomaine",...
			   "indice_min",1,...
			   "indice_max",6);
maillages_carre_Dir($+1)=liste_maillages($).nom;

liste_maillages($+1)=struct("nom","maillage_local_raffine",...
			   "indice_min",1,...
			   "indice_max",5);
maillages_carre_Dir($+1)=liste_maillages($).nom;

liste_maillages($+1)=struct("nom","maillage_triangle",...
			   "indice_min",1,...
			   "indice_max",6);
maillages_carre_Dir($+1)=liste_maillages($).nom;
			   
liste_maillages($+1)=struct("nom","maillage_deforme",...
			   "indice_min",1,...
			   "indice_max",5);
maillages_carre_Dir($+1)=liste_maillages($).nom;		
			   
liste_maillages($+1)=struct("nom","maillage_triangle_centregravite",...
			   "indice_min",1,...
			   "indice_max",6);
maillages_carre_Dir($+1)=liste_maillages($).nom;


/////////////////////////////////////////////////////////
/////   Les maillages d'un domaine fracture avec un écoulement vertical
/////////////////////////////////////////////////////////

maillages_fractures=list();

liste_maillages($+1)=struct("nom","maillage_2fractures",...
			    "courant","oui");
maillages_fractures($+1)=liste_maillages($).nom;

liste_maillages($+1)=struct("nom","maillage_3fractures",...
			    "courant","oui");
maillages_fractures($+1)=liste_maillages($).nom;

liste_maillages($+1)=struct("nom","maillage_3fractures_bis",...
			    "courant","oui");
maillages_fractures($+1)=liste_maillages($).nom;


//**************************************************************************
//   Fonction de lecture d'un maillage sur le disque dur
//**************************************************************************

function [maillage]=lecture_maillage(nom)
  
   [sommets,text_som]=fscanfMat(nom+'_sommets');
   [centres,text_cen]=fscanfMat(nom+'_centres');
   [aretes,text_aretes]=fscanfMat(nom+'_aretes');

   maillage=struct("nom",text_aretes(1),...
                   "sommets",sommets,...
		  "centres",centres,...
		   "aretes",aretes,...
		   "nb_vol",size(centres,1),...
		   "nb_are",size(aretes,1),...
		   "nb_som",size(sommets,1));
endfunction

//**************************************************************************
//   Fonction d'évaluation d'une fonction de deux variables sur une 
//     famille de points
//
//    points : une matrice  N x 2
//         f : une fonction (x,y) -> z
//
//   NB : Cette fonction n'est pas du tout optimisée !
//**************************************************************************

function [v]=eval_fonction(points,f)
    v=zeros(size(points,1),1);
    for i=1:size(points,1)
       v(i)=f(points(i,_X),points(i,_Y));
    end;
endfunction


//**************************************************************************
//   Fonction de tracé d'une fonction discrète définie sur les centres
//
//   m : un maillage
//   f : un vecteur colonne de taille m.nb_vol
//   nwin : numero de la fenetre graphique
//   ep_maillage : epaisseur des traits du maillage (=0 pour ne pas afficher le maillage)
//**************************************************************************

function trace_fonction(m,f,nwin,ep_maillage)
  
  fmin=min(f);
  fmax=max(f);
  
  scf(nwin);
  hf=gcf();
  clf;
  hf.color_map=jetcolormap(64);
  colorbar(fmin,fmax);
  hf.pixmap='on';
  
    s_X=zeros(3*m.nb_are,1);
    s_X(1:3:$)=m.centres(m.aretes(:,_K),_X);
    s_X(2:3:$)=m.sommets(m.aretes(:,_DEB),_X);
    s_X(3:3:$)=m.sommets(m.aretes(:,_FIN),_X);
    
    s_Y=zeros(3*m.nb_are,1);
    s_Y(1:3:$)=m.centres(m.aretes(:,_K),_Y);
    s_Y(2:3:$)=m.sommets(m.aretes(:,_DEB),_Y);
    s_Y(3:3:$)=m.sommets(m.aretes(:,_FIN),_Y);
    
    tri=zeros(m.nb_are,5);
    tri(:,2)=[1:3:3*m.nb_are]';
    tri(:,3)=[2:3:3*m.nb_are]';
    tri(:,4)=[3:3:3*m.nb_are]';
    
    val=zeros(3*m.nb_are,1);
    val(1:3:3*m.nb_are)=f(m.aretes(:,_K));
    val(2:3:3*m.nb_are)=val(1:3:3*m.nb_are);
    val(3:3:3*m.nb_are)=val(1:3:3*m.nb_are);
    
    fec(s_X',s_Y',tri,val',zminmax=[fmin,fmax]);
    
    temp=find(m.aretes(:,_L)>0);
    
    nb=length(temp);
    
    s_X=zeros(3*nb,1);
    s_X(1:3:$)=m.centres(m.aretes(temp,_L),_X);
    s_X(2:3:$)=m.sommets(m.aretes(temp,_DEB),_X);
    s_X(3:3:$)=m.sommets(m.aretes(temp,_FIN),_X);
    
    s_Y=zeros(3*nb,1);
    s_Y(1:3:$)=m.centres(m.aretes(temp,_L),_Y);
    s_Y(2:3:$)=m.sommets(m.aretes(temp,_DEB),_Y);
    s_Y(3:3:$)=m.sommets(m.aretes(temp,_FIN),_Y);
    
    tri=zeros(nb,5);
    tri(:,2)=[1:3:3*nb]';
    tri(:,3)=[2:3:3*nb]';
    tri(:,4)=[3:3:3*nb]';
    
    val=zeros(3*nb,1);
    val(1:3:3*nb)=f(m.aretes(temp,_L));
    val(2:3:3*nb)=val(1:3:3*nb);
    val(3:3:3*nb)=val(1:3:3*nb);
    
    fec(s_X',s_Y',tri,val',zminmax=[fmin,fmax]);
        
    if ep_maillage>0 then
    
      s_X=zeros(2*m.nb_are,1);
      s_Y=zeros(2*m.nb_are,1);
      s_X(1:2:$)=m.sommets(m.aretes(:,_DEB),_X);
      s_Y(1:2:$)=m.sommets(m.aretes(:,_DEB),_Y);
      s_X(2:2:$)=m.sommets(m.aretes(:,_FIN),_X);
      s_Y(2:2:$)=m.sommets(m.aretes(:,_FIN),_Y);
    
      xsegs(s_X,s_Y);
      e=gce();
      e.thickness=ep_maillage;
    end;

    
    show_pixmap();
    hf.pixmap='off';
endfunction


//**************************************************************************
//  Calcul de la norme L2 d'une fonction discrète définie sur les centres des volumes
//  m : un maillage
//  f : un vecteur colonne de taille m.nb_vol
//**************************************************************************

function [norme]=norme_L2(m,f)
  norme=0;
  for i=1:m.nb_are
     norme = norme + (m.aretes(i,_MES_K_DEB)+m.aretes(i,_MES_K_FIN))* ...
	     f(m.aretes(i,_K))^2;
     if (m.aretes(i,_L)>0) then
        norme = norme + (m.aretes(i,_MES_L_DEB)+m.aretes(i,_MES_L_FIN))* ...
	     f(m.aretes(i,_L))^2;
     end;
  end;
  norme=sqrt(norme);
endfunction

//**************************************************************************
//  Calcul de la norme H^1_0 d'une fonction discrète définie sur les centres des volumes
//  m : un maillage
//  f : un vecteur colonne de taille m.nb_vol
//
//  NB : pour DDFV, cette fonction ne calcule qu'une partie de la norme H1 !!
//**************************************************************************

function [norme]=norme_H1(m,f)
  norme=0;
  gf=zeros(m.nb_are,1);
  for i=1:m.nb_are
     gf(i) =  f(m.aretes(i,_K));
     if (m.aretes(i,_L)>0) then
       gf(i) = gf(i) - f(m.aretes(i,_L));
     end;
     gf(i)=gf(i)/m.aretes(i,_DKL);
  end;
  norme=sqrt(sum(...
      (m.aretes(:,_MES_K_DEB)+m.aretes(:,_MES_K_FIN)+m.aretes(:,_MES_L_DEB)+m.aretes(:,_MES_L_FIN))...
      .* (gf(:).^2)));
endfunction
