//////////////////////////////////////////////////////////////
/////           Divers schemas de volumes finis        ///////
/////               pour pbs elliptiques 2D            ///////
/////                                                  ///////
/////              différents schémas de VF            ///////
/////                                                  ///////
/////                F. Boyer  &  S. Krell             ///////
/////                     Mars 2010                    ///////
//////////////////////////////////////////////////////////////

// Toutes ces fonctions prennent deux paramètres :
//      m        : un maillage
//      donnees  : un jeu de donnees
//

//**************************************************************************
//    Routine de calcul du coefficient de diffusion aux aretes
//        selon la méthode choisie
//**************************************************************************
function [ck]=calcul_coeff_k_interf(m,donnees);
  
  if ~isfield(donnees,"methode") then
    donnees.methode="exacte";
  end;

  centres_aretes=(m.sommets(m.aretes(:,_DEB),:)+m.sommets(m.aretes(:,_FIN),:))/2;
  select donnees.methode
   case "exacte"  // On fait le calcul exact du coefficient aux centres
                  // des aretes
    
     ck=eval_fonction(centres_aretes,donnees.coeff_k);
     
   case "arithmetique"   // On fait la moyenne arithmétique des valeurs du
                         // coefficient aux centres des mailles voisines
  
     ck=eval_fonction(m.centres(m.aretes(:,_K),[_X _Y]),donnees.coeff_k);
     
     temp=find(m.aretes(:,_L)>0);
     ck(temp)=0.5*(ck(temp) ...
		   + eval_fonction(m.centres(m.aretes(temp,_L),[_X _Y]),donnees.coeff_k));
     
   case "harmonique"   // On fait la moyenne harmonique pondérée des
                       // valeurs du coefficient aux centres des mailles voisines
 
     ck=eval_fonction(m.centres(m.aretes(:,_K),[_X _Y]),donnees.coeff_k);
     
     temp=find(m.aretes(:,_L)>0);
     ckL=eval_fonction(m.centres(m.aretes(temp,_L),[_X _Y]),donnees.coeff_k);
     
     ck(temp)=  m.aretes(temp,_DKL).*ck(temp).*ckL ...
	      ./(m.aretes(temp,_DKsigma).*ckL+m.aretes(temp,_DLsigma).*ck(temp));
  else
     disp("Vous n''avez pas choisi une methode de calcul du coeff de"+...
	  " diffusion valable!");
     halt;
  end;
endfunction



//**************************************************************************
//    Le schéma VF4 avec conditions de Dirichlet homogène
//    programmé à l'aide de boucles
//    Celui-ci n'est pas utilisé mais est seulement présenté
//    pour des raisons pédagogiques
//**************************************************************************

function [A,b]=const_schema_VF4_boucles(m,donnees)
  
  source=eval_fonction(centres,donnees.source);
  
  A=spzeros(m.nb_vol,m.nb_vol);
  b=zeros(m.nb_vol,1);
  
  
  for i=1:m.nb_are
    
    centre_arete(_X)=(m.sommets(m.aretes(i,_DEB),_X)+m.sommets(m.aretes(i,_FIN),_X))/2;
    centre_arete(_Y)=(m.sommets(m.aretes(i,_DEB),_Y)+m.sommets(m.aretes(i,_FIN),_Y))/2;
    coeff_diff=donnees.coeff_k(centre_arete(_X),centre_arete(_Y));
    tauKL = coeff_diff*m.aretes(i,_MES)/m.aretes(i,_DKL);
    
    A(m.aretes(i,_K),m.aretes(i,_K)) = A(m.aretes(i,_K),m.aretes(i,_K)) + tauKL;
    
    b(m.aretes(i,_K)) = b(m.aretes(i,_K)) ...
	+ (m.aretes(i,_MES_K_DEB)+m.aretes(i,_MES_K_FIN))...
	   * donnees.source(m.centres(m.aretes(i,_K),_X) , m.centres(m.aretes(i,_K),_Y));
    
    
    if (m.aretes(i,_L)>0) then
      A(m.aretes(i,_K),m.aretes(i,_L)) = - tauKL;
      A(m.aretes(i,_L),m.aretes(i,_L)) = A(m.aretes(i,_L),m.aretes(i,_L)) + tauKL;
      A(m.aretes(i,_L),m.aretes(i,_K)) = - tauKL;

      b(m.aretes(i,_L)) = b(m.aretes(i,_L)) ...
	+ (m.aretes(i,_MES_L_DEB)+m.aretes(i,_MES_L_FIN))...
	   * donnees.source(m.centres(m.aretes(i,_L),_X) , m.centres(m.aretes(i,_L),_Y));      
    end;

  end;

endfunction


//**************************************************************************
//    Le schéma VF4 
//    programmé à l'aide d'opérations vectorielles pour des raisons d'efficacité 
//    Prise en compte des CL de Dirichlet non-homogène (_LABEL=-1)
//                    des CL de Neumann homogène  (_LABEL<-1)
//    Prise en compte de fractures à l'intérieur du domaine (_LABEL=1)
//**************************************************************************

function [A,b]=const_schema_VF4(m,donnees)

  printf("Assemblage VF4\n");
  
  // On évalue le terme source aux centres des mailles
  
  source=eval_fonction(m.centres,donnees.source);
  
  // On calcule le coefficient de diffusion (scalaire) sur chaque arête
  
  coeff_diff=calcul_coeff_k_interf(m,donnees);
  
  indiA=[];
  indjA=[];
  valA=[];
  
  indib=[];
  indjb=[];
  valb=[];
  
  // Calcul de la transmissivite de chaque arête 
  
  tauKL=coeff_diff(:).*m.aretes(:,_MES)./m.aretes(:,_DKL);

  // On corrige la transmissivite en cas de fissures dans le maillage
  if isfield(donnees,"beta") then
    
       // Recherche des aretes labellisees !  
             temp=find(m.aretes(:,_LABEL)==1);
             tauKL(temp)=donnees.beta*tauKL(temp).*m.aretes(temp,_DKL)...
		          ./(1+donnees.beta*m.aretes(temp,_DKL));
 
  end;
  
  /// Les aretes du bord

  /// Recherche des arêtes pour la condition de Dirichlet non homogène
  temp=find(m.aretes(:,_LABEL)==-1);

  // Evaluation des données au bord
  ubord=eval_fonction(...
      0.5*(m.sommets(m.aretes(temp,_DEB),[_X _Y])+m.sommets(m.aretes(temp,_FIN),[_X _Y]))...
      ,donnees.bordD);

  indiA=m.aretes(temp,_K);
  indjA=indiA;
  valA=tauKL(temp);                     

  indib=[indib; m.aretes(temp,_K)];
  valb=[valb; tauKL(temp).*ubord];


  /// Toutes les autres arêtes du bord sont supposées 
  /// correspondre à des CL de Neumann non homogène
  /// Donc elles ne contribuent pas au schéma
  
  
  /// La partie du terme source qui concerne tout le monde
  
  indib=[indib; m.aretes(:,_K)];
  valb=[valb; (m.aretes(:,_MES_K_DEB)+m.aretes(:,_MES_K_FIN)).*source(m.aretes(:,_K))];

  /// On traite maintenant les aretes interieures
  
  temp=find(m.aretes(:,_L)>0);

  indiA=[indiA; m.aretes(temp,_K)];
  indjA=[indjA; m.aretes(temp,_K)];
  valA=[valA; tauKL(temp)];  
  
  indiA=[indiA; m.aretes(temp,_K)];
  indjA=[indjA; m.aretes(temp,_L)];
  valA=[valA; -tauKL(temp)];
  
  indiA=[indiA; m.aretes(temp,_L)];
  indjA=[indjA; m.aretes(temp,_K)];
  valA=[valA; -tauKL(temp)];  

  indiA=[indiA; m.aretes(temp,_L)];
  indjA=[indjA; m.aretes(temp,_L)];
  valA=[valA; tauKL(temp)];  
  
  indib=[indib; m.aretes(temp,_L)];
  valb=[valb; (m.aretes(temp,_MES_L_DEB)+m.aretes(temp,_MES_L_FIN)).*source(m.aretes(temp,_L))];


  /// Construction proprement dite
  
  indjb=ones(indib);  
  A=sparse([indiA indjA], valA);
  b=sparse([indib indjb], valb);
  b=full(b);
  
endfunction


//**************************************************************************
//    Le schéma DDFV pour une diffusion scalaire
//    programmé à l'aide d'opérations vectorielles
//    Prise en compte des CL de Dirichlet non-homogène (_LABEL=-1)
//                    des CL de Neumann homogène  (_LABEL<-1)
//**************************************************************************


function [A,b]=const_schema_DDFV(m,donnees)

  printf("Assemblage DDFV\n");

  // On évalue le terme source aux centres et aux sommets
  source_centres=eval_fonction(m.centres,donnees.source);
  source_sommets=eval_fonction(m.sommets,donnees.source);
  
  centres_aretes=(m.sommets(m.aretes(:,_DEB),:)+m.sommets(m.aretes(:,_FIN),:))/2;
  
  // On calcule la valeur du coefficient de diffusion au centre de l'arête primale
  coeff_diff=calcul_coeff_k_interf(m,donnees);
  
  // On calcule la mesure des diamants
  md=m.aretes(:,_MES_K_DEB)+m.aretes(:,_MES_K_FIN)+m.aretes(:,_MES_L_DEB)+m.aretes(:,_MES_L_FIN);

  // On calcule le produit scalaire entre la normale primale et la normale duale
  // Ce coefficient est nul si le maillage est orthogonal
  N_KLscaNS_KL=m.aretes(:,_N_KL_X).*m.aretes(:,_NS_DEBFIN_X)+m.aretes(:,_N_KL_Y).*m.aretes(:,_NS_DEBFIN_Y);
  
  // On calcule maintenant les coefficients qui serviront dans la construction du schéma
  flux_NN=coeff_diff(:).*0.5.*m.aretes(:,_MES).*m.aretes(:,_MES)./md;
  flux_NNS=coeff_diff(:).*0.5.*m.aretes(:,_MES).*m.aretes(:,_DKL)./md.*N_KLscaNS_KL;
  flux_NSN=flux_NNS;
  flux_NSNS=coeff_diff(:).*0.5.*m.aretes(:,_DKL).*m.aretes(:,_DKL)./md;
  
  // Ces coefficients interviennent de la façon suivante
  // Dans le flux numérique
  // m.aretes(:,_MES)*k_sig*grad(u).N_KL=flux_NN*(u_L-u_K)+flux_NNS*(u_FIN-u_DEB)
  // m.aretes(:,_DKL)*k_sig*grad(u).NS_KL=flux_NSN*(u_L-u_K)+flux_NSNS*(u_FIN-u_DEB)
  
  // On a besoin de repérer les sommets au bord pour assembler le terme source
  // som_au_bord vaudra 1 a l'interieur et 0 au bord

  som_au_bord=ones(m.nb_som,1);                             
  temp=find(m.aretes(:,_L)<=0);
  som_au_bord(m.aretes(temp,_DEB))=0;
  som_au_bord(m.aretes(temp,_FIN))=0;
  
  // On évalue la donnée de Dirichlet sur tous les sommets
  
  ub_som=zeros(m.nb_som,1); 
  ub_som=eval_fonction(m.sommets(:,[_X _Y]),donnees.bordD);     
 
  // Début de la construction de la matrice et du second membre
  
  indiA=[];
  indjA=[];
  valA=[];
  
  indib=[];
  indjb=[];
  valb=[];
  
  // On cherche les arêtes du bord Dirichlet non homogène
  temp=find(m.aretes(:,_LABEL)==-1);

  // On évalue la donnée de Dirichlet au centre de ces arêtes
  ubord=eval_fonction(centres_aretes(temp,:),donnees.bordD);
  
  
  indiA=m.aretes(temp,_K);
  indjA=indiA;
  valA=flux_NN(temp);                     

  indib=m.aretes(temp,_K);
  valb=flux_NN(temp).*ubord - flux_NNS(temp).*ub_som(m.aretes(temp,_DEB)) ...
                            + flux_NNS(temp).*ub_som(m.aretes(temp,_FIN));
  
  // Les autres arêtes du bord sont supposées correspondre a une condition
  // de Neumann homogène qui ne nécessite aucun traitement particulier
  
  
  // On traite maintenant les arêtes intérieures.
  
  temp=find(m.aretes(:,_L)>0);
  
  // Contributions à l'équation de la maille K
  
  indiA=[indiA;m.aretes(temp,_K)];
  indjA=[indjA;m.aretes(temp,_K)];
  valA=[valA;flux_NN(temp)];
  
  indiA=[indiA; m.aretes(temp,_K)];
  indjA=[indjA; m.aretes(temp,_L)];
  valA=[valA; -flux_NN(temp)];
  
  indiA=[indiA; m.aretes(temp,_K)];
  indjA=[indjA; m.aretes(temp,_DEB)+m.nb_vol];
  valA=[valA; flux_NNS(temp)];
  
  indiA=[indiA; m.aretes(temp,_K)];
  indjA=[indjA; m.aretes(temp,_FIN)+m.nb_vol];
  valA=[valA; -flux_NNS(temp)];  
  
  indib=[indib;m.aretes(:,_K)];
  valb=[valb; (m.aretes(:,_MES_K_DEB)+m.aretes(:,_MES_K_FIN)).*source_centres(m.aretes(:,_K))]; 
  
  // Contributions à l'équation de la maille L
  
  indiA=[indiA; m.aretes(temp,_L)];
  indjA=[indjA; m.aretes(temp,_K)];
  valA=[valA; -flux_NN(temp)];  

  indiA=[indiA; m.aretes(temp,_L)];
  indjA=[indjA; m.aretes(temp,_L)];
  valA=[valA; flux_NN(temp)];
    
  indiA=[indiA; m.aretes(temp,_L)];
  indjA=[indjA; m.aretes(temp,_DEB)+m.nb_vol];
  valA=[valA; -flux_NNS(temp)];
  
  indiA=[indiA; m.aretes(temp,_L)];
  indjA=[indjA; m.aretes(temp,_FIN)+m.nb_vol];
  valA=[valA; flux_NNS(temp)];
  
  indib=[indib; m.aretes(temp,_L)];
  valb=[valb; (m.aretes(temp,_MES_L_DEB)+m.aretes(temp,_MES_L_FIN)).*source_centres(m.aretes(temp,_L))];

  // Contributions à l'équation du sommet DEB
  // En prenant garde aux sommets du bord
  
  indiA=[indiA; m.aretes(:,_DEB)+m.nb_vol];
  indjA=[indjA; m.aretes(:,_DEB)+m.nb_vol];
  valA=[valA; flux_NSNS(:).*som_au_bord(m.aretes(:,_DEB))+1-som_au_bord(m.aretes(:,_DEB))];
  
  indiA=[indiA; m.aretes(temp,_DEB)+m.nb_vol];
  indjA=[indjA; m.aretes(temp,_FIN)+m.nb_vol];
  valA=[valA; -flux_NSNS(temp).*som_au_bord(m.aretes(temp,_DEB))];
  
  indiA=[indiA; m.aretes(temp,_DEB)+m.nb_vol];
  indjA=[indjA; m.aretes(temp,_K)];
  valA=[valA; flux_NSN(temp).*som_au_bord(m.aretes(temp,_DEB))];
  
  indiA=[indiA; m.aretes(temp,_DEB)+m.nb_vol];
  indjA=[indjA; m.aretes(temp,_L)];
  valA=[valA; -flux_NSN(temp).*som_au_bord(m.aretes(temp,_DEB))];
  
  // Le second membre est différent si on est au bord ou a l'interieur
  test= (m.aretes(:,_MES_K_DEB)+m.aretes(:,_MES_L_DEB)).*source_sommets(m.aretes(:,_DEB)) ...
           .*som_au_bord(m.aretes(:,_DEB)) ...
       +(1-som_au_bord(m.aretes(:,_DEB))).*ub_som(m.aretes(:,_DEB)) ;
     
    
  indib=[indib; m.aretes(:,_DEB)+m.nb_vol];
  valb=[valb;test ];
 
  // Contributions à l'équation du sommet FIN
  // En prenant garde aux sommets du bord
  
  indiA=[indiA; m.aretes(:,_FIN)+m.nb_vol];
  indjA=[indjA; m.aretes(:,_FIN)+m.nb_vol];
  valA=[valA; flux_NSNS(:).*som_au_bord(m.aretes(:,_FIN))+1-som_au_bord(m.aretes(:,_FIN))];
  
  
  indiA=[indiA; m.aretes(temp,_FIN)+m.nb_vol];
  indjA=[indjA; m.aretes(temp,_DEB)+m.nb_vol];
  valA=[valA; -flux_NSNS(temp).*som_au_bord(m.aretes(temp,_FIN))];
  
  indiA=[indiA; m.aretes(temp,_FIN)+m.nb_vol];
  indjA=[indjA; m.aretes(temp,_K)];
  valA=[valA; -flux_NSN(temp).*som_au_bord(m.aretes(temp,_FIN))];
  
  indiA=[indiA; m.aretes(temp,_FIN)+m.nb_vol];
  indjA=[indjA; m.aretes(temp,_L)];
  valA=[valA; flux_NSN(temp).*som_au_bord(m.aretes(temp,_FIN))];
  
  
  // Le second membre est différent si on est au bord ou a l'interieur
  test= (m.aretes(:,_MES_K_FIN)+m.aretes(:,_MES_L_FIN)).*source_sommets(m.aretes(:,_FIN)) ...
         .*som_au_bord(m.aretes(:,_FIN)) ...
       +(1-som_au_bord(m.aretes(:,_FIN))).*ub_som(m.aretes(:,_FIN));
  
  indib=[indib; m.aretes(:,_FIN)+m.nb_vol];
  valb=[valb; test];
  
  
  // Construction proprement dite
  
  indjb=ones(indib);
  
  A=sparse([indiA indjA], valA);
  b=sparse([indib indjb], valb);
  b=full(b);
  
endfunction


//**************************************************************************
//    Le schéma DDFV pour une diffusion matricielle (non isotrope)
//    programmé à l'aide d'opérations vectorielles
//    Prise en compte des CL de Dirichlet non-homogène (_LABEL=-1)
//                    des CL de Neumann homogène  (_LABEL<-1)
//**************************************************************************

function [A,b]=const_schema_DDFV_mat(m,donnees)

  printf("Assemblage DDFV Anisotrope\n");

  // On évalue le terme source aux centres et aux sommets
  source_centres=eval_fonction(m.centres,donnees.source);
  source_sommets=eval_fonction(m.sommets,donnees.source);
  
  centres_aretes=( m.sommets(m.aretes(:,_DEB),:)...
		  +m.sommets(m.aretes(:,_FIN),:))/2;
  
  
  // Dans le cas ou la matrice est isotrope (donnee par coeff_k)
  // On construit explicitement la matrice dans coeff_mat
  
  if ~isfield(donnees,"coeff_mat") then
    deff('[z]=temp_axx(x,y)','z=donnees.coeff_k(x,y)');
    deff('[z]=temp_axy(x,y)','z=0');
    deff('[z]=temp_ayy(x,y)','z=donnees.coeff_k(x,y)');
    donnees.coeff_mat=struct('Axx',temp_axx,'Axy',temp_axy,'Ayy',temp_ayy);
  end;
  
  // On calcule la valeur du coefficient de diffusion
  // par l'une des méthodes disponibles
  
    [ADNN,ADNNS,ADNSNS]=calcul_coeff_mat(m,donnees); 
    
  
  // On calcule la mesure des diamants
  md=m.aretes(:,_MES_K_DEB)+m.aretes(:,_MES_K_FIN)+m.aretes(:,_MES_L_DEB)+m.aretes(:,_MES_L_FIN);
  
  // On calcule maintenant les coefficients qui serviront dans la construction du schéma
  flux_NN=0.5.*m.aretes(:,_MES).*m.aretes(:,_MES).*ADNN./md;
  flux_NNS=0.5.*m.aretes(:,_MES).*m.aretes(:,_DKL).*ADNNS./md;
  flux_NSN=flux_NNS;
  flux_NSNS=0.5.*m.aretes(:,_DKL).*m.aretes(:,_DKL).*ADNSNS./md;
  
  // Ces coefficients interviennent de la façon suivante :
  // dans la définition du schéma numérique
  // m.aretes(:,_MES)*AD*grad(u).N_KL=flux_NN*(u_L-u_K)+flux_NNS*(u_FIN-u_DEB)
  // m.aretes(:,_DKL)*AD*grad(u).NS_KL=flux_NSN*(u_L-u_K)+flux_NSNS*(u_FIN-u_DEB)
  
  // On a besoin de repérer les sommets au bord pour assembler le terme source
  // som_au_bord vaudra 1 a l'interieur et 0 au bord

  som_au_bord=ones(m.nb_som,1);                             
  temp=find(m.aretes(:,_L)<=0);
  som_au_bord(m.aretes(temp,_DEB))=0;
  som_au_bord(m.aretes(temp,_FIN))=0;
  
  // On évalue la donnée de Dirichlet sur tous les sommets
  
  ub_som=zeros(m.nb_som,1); 
  ub_som=eval_fonction(m.sommets(:,[_X _Y]),donnees.bordD);     
 
  // Début de la construction de la matrice et du second membre
  
  indiA=[];
  indjA=[];
  valA=[];
  
  indib=[];
  indjb=[];
  valb=[];
  
  // On cherche les arêtes du bord Dirichlet non homogène
  temp=find(m.aretes(:,_LABEL)==-1);

  // On évalue la donnée de Dirichlet au centre de ces arêtes
  ubord=eval_fonction(centres_aretes(temp,:),donnees.bordD);
  
  
  indiA=m.aretes(temp,_K);
  indjA=indiA;
  valA=flux_NN(temp);                     

  indib=m.aretes(temp,_K);
  valb=flux_NN(temp).*ubord - flux_NNS(temp).*ub_som(m.aretes(temp,_DEB)) ...
                            + flux_NNS(temp).*ub_som(m.aretes(temp,_FIN));
  
  // Les autres arêtes du bord sont supposées correspondre a une condition
  // de Neumann homogène qui ne nécessite aucun traitement particulier
  
  
  // On traite maintenant les arêtes intérieures.
  
  temp=find(m.aretes(:,_L)>0);
  
  // Contributions à l'équation de la maille K
  
  indiA=[indiA;m.aretes(temp,_K)];
  indjA=[indjA;m.aretes(temp,_K)];
  valA=[valA;flux_NN(temp)];
  
  indiA=[indiA; m.aretes(temp,_K)];
  indjA=[indjA; m.aretes(temp,_L)];
  valA=[valA; -flux_NN(temp)];
  
  indiA=[indiA; m.aretes(temp,_K)];
  indjA=[indjA; m.aretes(temp,_DEB)+m.nb_vol];
  valA=[valA; flux_NNS(temp)];
  
  indiA=[indiA; m.aretes(temp,_K)];
  indjA=[indjA; m.aretes(temp,_FIN)+m.nb_vol];
  valA=[valA; -flux_NNS(temp)];  
  
  indib=[indib;m.aretes(:,_K)];
  valb=[valb; (m.aretes(:,_MES_K_DEB)+m.aretes(:,_MES_K_FIN)).*source_centres(m.aretes(:,_K))]; 
 
  // Contributions à l'équation de la maille L
  
  indiA=[indiA; m.aretes(temp,_L)];
  indjA=[indjA; m.aretes(temp,_K)];
  valA=[valA; -flux_NN(temp)];  

  indiA=[indiA; m.aretes(temp,_L)];
  indjA=[indjA; m.aretes(temp,_L)];
  valA=[valA; flux_NN(temp)];
    
  indiA=[indiA; m.aretes(temp,_L)];
  indjA=[indjA; m.aretes(temp,_DEB)+m.nb_vol];
  valA=[valA; -flux_NNS(temp)];
  
  indiA=[indiA; m.aretes(temp,_L)];
  indjA=[indjA; m.aretes(temp,_FIN)+m.nb_vol];
  valA=[valA; flux_NNS(temp)];
  
  indib=[indib; m.aretes(temp,_L)];
  valb=[valb; (m.aretes(temp,_MES_L_DEB)+m.aretes(temp,_MES_L_FIN)).*source_centres(m.aretes(temp,_L))];

  // Contributions à l'équation du sommet DEB
  // En prenant garde aux sommets du bord
   
  indiA=[indiA; m.aretes(:,_DEB)+m.nb_vol];
  indjA=[indjA; m.aretes(:,_DEB)+m.nb_vol];
  valA=[valA; flux_NSNS(:).*som_au_bord(m.aretes(:,_DEB))+ones(m.nb_are,1)-som_au_bord(m.aretes(:,_DEB))];
  
  indiA=[indiA; m.aretes(temp,_DEB)+m.nb_vol];
  indjA=[indjA; m.aretes(temp,_FIN)+m.nb_vol];
  valA=[valA; -flux_NSNS(temp).*som_au_bord(m.aretes(temp,_DEB))];
  
  indiA=[indiA; m.aretes(temp,_DEB)+m.nb_vol];
  indjA=[indjA; m.aretes(temp,_K)];
  valA=[valA; flux_NSN(temp).*som_au_bord(m.aretes(temp,_DEB))];
  
  indiA=[indiA; m.aretes(temp,_DEB)+m.nb_vol];
  indjA=[indjA; m.aretes(temp,_L)];
  valA=[valA; -flux_NSN(temp).*som_au_bord(m.aretes(temp,_DEB))];
  
  // Le second membre est différent si on est au bord ou a l'interieur
  test= (m.aretes(:,_MES_K_DEB)+m.aretes(:,_MES_L_DEB)).*source_sommets(m.aretes(:,_DEB)) ...
           .*som_au_bord(m.aretes(:,_DEB)) ...
       +(1-som_au_bord(m.aretes(:,_DEB))).*ub_som(m.aretes(:,_DEB)) ;
     
    
  indib=[indib; m.aretes(:,_DEB)+m.nb_vol];
  valb=[valb;test ];
  
  // Contributions à l'équation du sommet FIN
  // En prenant garde aux sommets du bord
  
  indiA=[indiA; m.aretes(:,_FIN)+m.nb_vol];
  indjA=[indjA; m.aretes(:,_FIN)+m.nb_vol];
  valA=[valA; flux_NSNS(:).*som_au_bord(m.aretes(:,_FIN))+ones(m.nb_are,1)-som_au_bord(m.aretes(:,_FIN))];
  
  
  indiA=[indiA; m.aretes(temp,_FIN)+m.nb_vol];
  indjA=[indjA; m.aretes(temp,_DEB)+m.nb_vol];
  valA=[valA; -flux_NSNS(temp).*som_au_bord(m.aretes(temp,_FIN))];
  
  indiA=[indiA; m.aretes(temp,_FIN)+m.nb_vol];
  indjA=[indjA; m.aretes(temp,_K)];
  valA=[valA; -flux_NSN(temp).*som_au_bord(m.aretes(temp,_FIN))];
  
  indiA=[indiA; m.aretes(temp,_FIN)+m.nb_vol];
  indjA=[indjA; m.aretes(temp,_L)];
  valA=[valA; flux_NSN(temp).*som_au_bord(m.aretes(temp,_FIN))];
  
  
  // Le second membre est différent si on est au bord ou a l'interieur
  test= (m.aretes(:,_MES_K_FIN)+m.aretes(:,_MES_L_FIN)).*source_sommets(m.aretes(:,_FIN)) ...
         .*som_au_bord(m.aretes(:,_FIN)) ...
       +(1-som_au_bord(m.aretes(:,_FIN))).*ub_som(m.aretes(:,_FIN));
  
  indib=[indib; m.aretes(:,_FIN)+m.nb_vol];
  valb=[valb; test];
  
  
  // Construction proprement dite
  
  indjb=ones(indib);
  
  A=sparse([indiA indjA], valA);
  b=sparse([indib indjb], valb);
  b=full(b);
  
endfunction

//**************************************************************************
//    Routine de calcul du tenseur de diffusion sur un diamant
//    selon la méthode choisie
//**************************************************************************
function [ADNN,ADNNS,ADNSNS]=calcul_coeff_mat(m,donnees);

  if ~isfield(donnees,'methode') then
    donnees.methode='exacte';
  end;
  
  centres_aretes=(m.sommets(m.aretes(:,_DEB),:)+m.sommets(m.aretes(:,_FIN),:))/2;
  
  ADxx=eval_fonction(centres_aretes,donnees.coeff_mat.Axx);
  ADxy=eval_fonction(centres_aretes,donnees.coeff_mat.Axy);
  ADyy=eval_fonction(centres_aretes,donnees.coeff_mat.Ayy);
  
  ADN_X=ADxx.*m.aretes(:,_N_KL_X)+ADxy.*m.aretes(:,_N_KL_Y);
  ADN_Y=ADxy.*m.aretes(:,_N_KL_X)+ADyy.*m.aretes(:,_N_KL_Y);
  
  ADNS_X=ADxx.*m.aretes(:,_NS_DEBFIN_X)+ADxy.*m.aretes(:,_NS_DEBFIN_Y);
  ADNS_Y=ADxy.*m.aretes(:,_NS_DEBFIN_X)+ADyy.*m.aretes(:,_NS_DEBFIN_Y);
  
  ADNN=ADN_X.*m.aretes(:,_N_KL_X)+ADN_Y.*m.aretes(:,_N_KL_Y);
  ADNNS=ADN_X.*m.aretes(:,_NS_DEBFIN_X)+ADN_Y.*m.aretes(:,_NS_DEBFIN_Y);
  ADNSNS=ADNS_X.*m.aretes(:,_NS_DEBFIN_X)+ADNS_Y.*m.aretes(:,_NS_DEBFIN_Y);
  
  AKxx=eval_fonction(m.centres,donnees.coeff_mat.Axx);
  AKxy=eval_fonction(m.centres,donnees.coeff_mat.Axy);
  AKyy=eval_fonction(m.centres,donnees.coeff_mat.Ayy);
  
  
  for i=1:m.nb_are
    if (m.aretes(i,_L)>0) then
      
      nbK=m.aretes(i,_K);
      nbL=m.aretes(i,_L);
      
      AKN_X=AKxx(nbK).*m.aretes(i,_N_KL_X)+AKxy(nbK).*m.aretes(i,_N_KL_Y);
      AKN_Y=AKxy(nbK).*m.aretes(i,_N_KL_X)+AKyy(nbK).*m.aretes(i,_N_KL_Y);
      
      AKNS_X=AKxx(nbK).*m.aretes(i,_NS_DEBFIN_X)+AKxy(nbK).*m.aretes(i,_NS_DEBFIN_Y);
      AKNS_Y=AKxy(nbK).*m.aretes(i,_NS_DEBFIN_X)+AKyy(nbK).*m.aretes(i,_NS_DEBFIN_Y);
      
      AKNN=AKN_X.*m.aretes(i,_N_KL_X)+AKN_Y.*m.aretes(i,_N_KL_Y);
      AKNNS=AKN_X.*m.aretes(i,_NS_DEBFIN_X)+AKN_Y.*m.aretes(i,_NS_DEBFIN_Y);
      AKNSNS=AKNS_X.*m.aretes(i,_NS_DEBFIN_X)+AKNS_Y.*m.aretes(i,_NS_DEBFIN_Y);
      
      ALN_X=AKxx(nbL).*m.aretes(i,_N_KL_X)+AKxy(nbL).*m.aretes(i,_N_KL_Y);
      ALN_Y=AKxy(nbL).*m.aretes(i,_N_KL_X)+AKyy(nbL).*m.aretes(i,_N_KL_Y);
      
      ALNS_X=AKxx(nbL).*m.aretes(i,_NS_DEBFIN_X)+AKxy(nbL).*m.aretes(i,_NS_DEBFIN_Y);
      ALNS_Y=AKxy(nbL).*m.aretes(i,_NS_DEBFIN_X)+AKyy(nbL).*m.aretes(i,_NS_DEBFIN_Y);
      
      ALNN=ALN_X.*m.aretes(i,_N_KL_X)+ALN_Y.*m.aretes(i,_N_KL_Y);
      ALNNS=ALN_X.*m.aretes(i,_NS_DEBFIN_X)+ALN_Y.*m.aretes(i,_NS_DEBFIN_Y);
      ALNSNS=ALNS_X.*m.aretes(i,_NS_DEBFIN_X)+ALNS_Y.*m.aretes(i,_NS_DEBFIN_Y);
      
      select donnees.methode
        
      case "arithmetique"   // On fait la moyenne arithmétique des valeurs du
                            // coefficient aux centres des mailles voisines
        
        ADNN(i)=0.5*(AKNN+ALNN);
        
        ADNNS(i)=0.5*(AKNNS+ALNNS);
        
        ADNSNS(i)=0.5*(AKNSNS+ALNSNS);
	
      case "harmonique"   // On fait la moyenne harmonique pondérée des
                          // valeurs du coefficient aux centres des mailles voisines
                          // selon la méthode m-DDFV

        ADNN(i)=((m.aretes(i,_DKsigma)+m.aretes(i,_DLsigma))*AKNN*ALNN) ...
                /(m.aretes(i,_DLsigma)*AKNN+m.aretes(i,_DKsigma)*ALNN);
        
        ADNNS(i)=(m.aretes(i,_DLsigma)*ALNNS*AKNN+ ...
                 +m.aretes(i,_DKsigma)*AKNNS*ALNN) ...
                /(m.aretes(i,_DLsigma)*AKNN+m.aretes(i,_DKsigma)*ALNN);
        
        ADNSNS(i)=(m.aretes(i,_DLsigma)*ALNSNS+m.aretes(i,_DKsigma)*AKNSNS) ...
                      /(m.aretes(i,_DKsigma)+m.aretes(i,_DLsigma)) ...
                  -m.aretes(i,_DLsigma)*m.aretes(i,_DKsigma)...
		     /(m.aretes(i,_DKsigma)+m.aretes(i,_DLsigma)) ...
                     *(AKNNS-ALNNS)^2/(m.aretes(i,_DLsigma)*AKNN+m.aretes(i,_DKsigma)*ALNN);
        
      end
    end
  end

endfunction

//**************************************************************************
//    Calcul de la fonction de courant associée à la solution
//    du problème elliptique calculé par VF4 sur un maillage
//    orthogonal admissible
//    Ne fonctionne pas avec DDFV !!!
//**************************************************************************

function [courant]=calcul_courant(m,donnees,u)
   
   centres_aretes=(m.sommets(m.aretes(:,_DEB),:)+m.sommets(m.aretes(:,_FIN),:))/2;
   coeff_diff=eval_fonction(centres_aretes,donnees.coeff_k);
   flux=zeros(m.nb_are,1);

   temp=find(m.aretes(:,_LABEL)>=0); // Aretes interieures
   flux(temp)=(u(m.aretes(temp,_L))-u(m.aretes(temp,_K)))./m.aretes(temp,_DKL).*m.aretes(temp,_MES);

   if isfield(donnees,'beta') then
      temp=find(m.aretes(:,_LABEL)==1); // Aretes correspondant à
                                        // d'éventuelles fractures
      flux(temp)=flux(temp)*donnees.beta.*m.aretes(temp,_DKL)./(1+ ...
						  donnees.beta*m.aretes(temp,_DKL));
   end;

   temp=find(m.aretes(:,_LABEL)==-1); // aretes du bord Dirichlet
   ubord=eval_fonction(m.sommets(m.aretes(temp,_DEB),[_X _Y]),donnees.bordD)
   flux(temp)=(ubord-u(m.aretes(temp,_K)))./m.aretes(temp,_DKL).*m.aretes(temp,_MES);
   


   // Pour eviter des problemes d'orientation

   signe= (m.sommets(m.aretes(:,_DEB),_X)-m.sommets(m.aretes(:,_FIN),_X)).*m.aretes(:,_N_KL_Y) ...
         -(m.sommets(m.aretes(:,_DEB),_Y)-m.sommets(m.aretes(:,_FIN),_Y)).*m.aretes(:,_N_KL_X);
   signe=signe./abs(signe);

   traite=zeros(m.nb_som,1);
   courant=zeros(m.nb_som,1);

   traite(1)=1;
   totaltraite=0;
   
   while (totaltraite<m.nb_som)
     for i=1:m.nb_are
      if ((traite(m.aretes(i,_DEB))==1) & (traite(m.aretes(i,_FIN))==0)) then
          traite(m.aretes(i,_FIN))=1;
          courant(m.aretes(i,_FIN))=courant(m.aretes(i,_DEB))+signe(i)*flux(i);
      end;

      if ((traite(m.aretes(i,_DEB))==0) & (traite(m.aretes(i,_FIN))==1)) then
          traite(m.aretes(i,_DEB))=1;
          courant(m.aretes(i,_DEB))=courant(m.aretes(i,_FIN))-signe(i)*flux(i);
      end;    
     end;

    totaltraite=sum(traite);
  end;

endfunction
 
//**************************************************************************
//    Routine d'interpolation de la fonction de courant (définie sur les sommets)
//    sur une grille cartésienne.
//     
//    Ceci est nécessaire pour tracer les lignes de courant dans Scilab
//
//    Ne marche que sur les triangles
//    et si le domaine est contenu dans le carré unité
//**************************************************************************  

function [phi]=interpole(m,courant,Nx,Ny);

   max_courant=max(courant);
   min_courant=min(courant);
  
   // On cherche les triangles
   
   tri=zeros(m.nb_vol,7);
   tri(:,1)=1;
   for i=1:m.nb_are
      j=m.aretes(i,_K);
      tri(j,tri(j,1)+1)=m.aretes(i,_DEB);
      tri(j,tri(j,1)+2)=m.aretes(i,_FIN);
      tri(j,1)=tri(j,1)+2;
      
      j=m.aretes(i,_L);
      if (j>0) then
	tri(j,tri(j,1)+1)=m.aretes(i,_DEB);
	tri(j,tri(j,1)+2)=m.aretes(i,_FIN);
	tri(j,1)=tri(j,1)+2;
      end;
   end;


   tri=tri(:,2:7)

   tri=gsort(tri,'c','i');
   tri=tri(:,[1 3 5]);


   x=linspace(0,1,Nx);
   y=linspace(0,1,Ny);
   [X Y] = meshgrid(x,y);
   phi=zeros(Nx,Ny);
   for i=1:size(X,1)
      for j=1:size(X,2)
	ptx=X(i,j);
	pty=Y(i,j);
	coord2= ( (ptx-m.sommets(tri(:,1),_X)).*(m.sommets(tri(:,3),_Y)-m.sommets(tri(:,1),_Y)) ...
                 -(pty-m.sommets(tri(:,1),_Y)).*(m.sommets(tri(:,3),_X)-m.sommets(tri(:,1),_X)))...
              ./(  (m.sommets(tri(:,2),_X)-m.sommets(tri(:,1),_X)).*(m.sommets(tri(:,3),_Y)-m.sommets(tri(:,1),_Y)) ...
                 -(m.sommets(tri(:,2),_Y)-m.sommets(tri(:,1),_Y)).*(m.sommets(tri(:,3),_X)-m.sommets(tri(:,1),_X)));
	coord3= ( (ptx-m.sommets(tri(:,1),_X)).*(m.sommets(tri(:,2),_Y)-m.sommets(tri(:,1),_Y)) ...
                 -(pty-m.sommets(tri(:,1),_Y)).*(m.sommets(tri(:,2),_X)-m.sommets(tri(:,1),_X)))...
              ./(  (m.sommets(tri(:,3),_X)-m.sommets(tri(:,1),_X)).*(m.sommets(tri(:,2),_Y)-m.sommets(tri(:,1),_Y)) ...
                 -(m.sommets(tri(:,3),_Y)-m.sommets(tri(:,1),_Y)).*(m.sommets(tri(:,2),_X)-m.sommets(tri(:,1),_X)));
	coord1=1-coord2-coord3;

	[temp]=find(abs(coord1)+abs(coord2)+abs(coord3)-1<10^(-10),1);

	if (length(temp)==0) then
	   phi(i,j)=max_courant+100*(max_courant-min_courant);
	else
	phi(i,j)=coord1(temp)*courant(tri(temp,1))+coord2(temp)*courant(tri(temp,2)) ...
                +coord3(temp)*courant(tri(temp,3));
	end;
      end;
   end;
   
   phi=phi';

endfunction